(function() {
  var Post, PostSchema, fs;
  fs = require('fs');
  PostSchema = new app.mongoose.Schema({
    title: String,
    topic: String,
    body: String,
    slug: String,
    date: Date
  });
  PostSchema.method('update', function(text) {
    var lines;
    lines = text.split('\n');
    this.title = lines.first();
    this.topic = lines[1];
    this.date = lines.last().toDate();
    this.body = lines.slice(2, lines.length - 1).join('\n');
    return this.save();
  });
  PostSchema.static('update_or_create', function(file, callback) {
    var slug, text;
    slug = file.name.split('.')[0];
    text = fs.readFileSync(file.path, 'utf-8');
    return Post.findOne({
      slug: slug
    }, function(err, post) {
      if (!post) {
        post = new Post;
        post.slug = slug;
      }
      post.update(text);
      return callback();
    });
  });
  PostSchema.static('topics', function(callback) {
    return Post.find({}, function(err, posts) {
      var topics;
      topics = [];
      Object.each(posts.groupBy('topic'), function(topic, list) {
        var desc;
        return topics.push({
          name: topic,
          posts: list.sortBy('date', desc = true)
        });
      });
      return callback(topics);
    });
  });
  Post = app.mongoose.model('Post', PostSchema);
  app.get('/', function(req, res) {
    return Post.topics(function(topics) {
      req.topics = topics;
      return res.render('index', req);
    });
  });
  app.get('/posts/:slug', function(req, res) {
    return Post.findOne({
      slug: req.params.slug
    }, function(err, post) {
      req.title = post.title;
      req.post = post;
      return res.render('post', req);
    });
  });
  app.post('/posts', function(req, res) {
    return req.form.complete(function(err, fields, files) {
      var file;
      if (err) {
        return next(err);
      } else {
        file = files.file;
        return Post.update_or_create(file, function() {
          return res.send('ok\n');
        });
      }
    });
  });
  app["delete"]('/posts/:slug', function(req, res) {
    return Post.findOne({
      slug: req.params.slug
    }, function(err, post) {
      if (post) {
        return post.remove(function() {
          return res.send("ok\n");
        });
      } else {
        return res.send("not found\n");
      }
    });
  });
  module.exports = Post;
}).call(this);
