(function() {
  var Template, TemplateSchema, block_end_regex, block_start_regex, clear_blocks, eco, fs, generate, get_name, http, include_regex, mongoose, name_regex, parent_regex, parse_includes, parse_parents, read_blocks, replace_blocks, res;
  var __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };
  eco = require('eco');
  mongoose = require('mongoose');
  fs = require('fs');
  name_regex = /"(\w|\d|_)+"/;
  block_start_regex = /<% ?block "(\w|\d|_)+" ?%>/g;
  block_end_regex = /<% ?endblock ?%>/g;
  parent_regex = /^<% ?parent "(\w|\d|_)+" ?%>/g;
  include_regex = /<% ?include "(\w|\d|_)+" ?%>/g;
  get_name = function(tag) {
    return name_regex.exec(tag)[0].replace('"', '').replace('"', '');
  };
  parse_includes = function(src, templates) {
    var tags;
    tags = src.match(include_regex);
    while (tags != null) {
      tags.each(function(tag) {
        var include_src, name;
        name = get_name(tag);
        include_src = templates[name] != null ? templates[name] : "<p><b>" + name + "</b> not found.</p>";
        return src = src.replace(tag, include_src);
      });
      tags = src.match(include_regex);
    }
    return src;
  };
  read_blocks = function(src) {
    var blocks, tags;
    blocks = {};
    tags = src.match(block_start_regex);
    if (tags) {
      tags.each(function(tag) {
        var content, end, name, start;
        name = get_name(tag);
        start = src.search(tag);
        end = start + src.substring(start).search(block_end_regex);
        content = src.substring(start, end).replace(tag, '');
        return blocks[name] = content;
      });
    }
    return blocks;
  };
  replace_blocks = function(src, blocks) {
    var tags;
    tags = src.match(block_start_regex);
    if (tags) {
      tags.each(function(tag) {
        var content, end, name, start;
        name = get_name(tag);
        start = src.search(tag);
        end = start + src.substring(start).search(block_end_regex);
        content = src.substring(start, end).replace(tag, '');
        if (blocks[name] != null) {
          return src = src.replace(content, blocks[name]);
        }
      });
    }
    return src;
  };
  clear_blocks = function(src) {
    return src.replace(block_start_regex, '').replace(block_end_regex, '');
  };
  parse_parents = function(src, templates) {
    var blocks, name, parent_blocks, parent_src, tags;
    blocks = read_blocks(src);
    tags = src.match(parent_regex);
    while (tags != null) {
      name = get_name(tags.first());
      parent_src = templates[name];
      parent_blocks = read_blocks(parent_src);
      blocks = Object.merge(parent_blocks, blocks);
      tags = parent_src.match(parent_regex);
    }
    if (parent_src != null) {
      src = parent_src;
    }
    src = replace_blocks(src, blocks);
    return src = clear_blocks(src);
  };
  generate = function(name, templates) {
    var src;
    src = templates[name];
    src = parse_parents(src, templates);
    return src = parse_includes(src, templates);
  };
  TemplateSchema = new mongoose.Schema({
    name: String,
    src: String
  });
  TemplateSchema.static('update_or_create', function(file, callback) {
    var name, src;
    name = file.name.replace('.html', '');
    src = fs.readFileSync(file.path, 'utf-8');
    return Template.findOne({
      name: name
    }, function(err, template) {
      if (!template) {
        template = new Template;
        template.name = name;
      }
      template.src = src;
      template.save();
      return callback();
    });
  });
  TemplateSchema.static('render', function(name, context, callback) {
    if (context == null) {
      context = {};
    }
    return Template.find({}, function(err, docs) {
      var html, src, templates;
      templates = {};
      docs.each(function(d) {
        return templates[d.name] = d.src;
      });
      src = generate(name, templates);
      try {
        html = eco.render(src, context);
      } catch (err) {
        html = "<b>Template:</b> " + err;
      }
      return callback(html);
    });
  });
  Template = mongoose.model('Template', TemplateSchema);
  app.post('/templates', function(req, res, next) {
    return req.form.complete(function(err, fields, files) {
      var file;
      if (err) {
        return next(err);
      } else {
        file = files.file;
        return Template.update_or_create(file, function() {
          return res.send('ok\n');
        });
      }
    });
  });
  app["delete"]('/templates/:name', function(req, res) {
    return Template.findOne({
      slug: req.params.name
    }, function(err, template) {
      if (template) {
        return template.remove(function() {
          return res.send("ok\n");
        });
      } else {
        return res.send("not found\n");
      }
    });
  });
  module.exports = Template;
  http = require('http');
  res = http.ServerResponse.prototype;
  res.render = function(name, context) {
    if (context == null) {
      context = {};
    }
    return Template.render(name, context, __bind(function(html) {
      return this.send(html);
    }, this));
  };
}).call(this);
