eco = require 'eco'
mongoose = require 'mongoose'
fs = require 'fs'


# REGEXs

name_regex = /"(\w|\d|_)+"/
block_start_regex = /<% ?block "(\w|\d|_)+" ?%>/g
block_end_regex = /<% ?endblock ?%>/g
parent_regex = /^<% ?parent "(\w|\d|_)+" ?%>/g
include_regex = /<% ?include "(\w|\d|_)+" ?%>/g


# HELPERS

get_name = (tag) -> name_regex.exec(tag)[0].replace('"', '').replace('"', '')

parse_includes = (src, templates) ->
  tags = src.match(include_regex)
  while tags?
    tags.each (tag) ->
      name = get_name tag
      include_src  = if templates[name]? then templates[name] else "<p><b>#{ name }</b> not found.</p>"
      src  = src.replace(tag, include_src) 
    tags = src.match(include_regex)
  src


read_blocks = (src) ->
  blocks = {}
  tags = src.match(block_start_regex)
  if tags
    tags.each (tag) ->    
      name          = get_name tag
      start         = src.search(tag)
      end           = start + src.substring(start).search(block_end_regex)
      content       = src.substring(start, end).replace(tag, '')
      blocks[name]  = content
  blocks


replace_blocks = (src, blocks) ->
  tags = src.match(block_start_regex)
  if tags
    tags.each (tag) ->
      name    = get_name tag
      start   = src.search(tag)
      end     = start + src.substring(start).search(block_end_regex)
      content = src.substring(start, end).replace(tag, '')

      src = src.replace content, blocks[name] if blocks[name]?
  src


clear_blocks = (src) ->
  src.replace(block_start_regex, '').replace(block_end_regex, '')


parse_parents = (src, templates) ->
  blocks = read_blocks(src)
  
  tags = src.match(parent_regex)
  while tags?
    name  = get_name tags.first()
    parent_src = templates[name]
    parent_blocks = read_blocks(parent_src)
    blocks = Object.merge( parent_blocks, blocks)
    tags = parent_src.match(parent_regex)

  src = parent_src if parent_src?
  src = replace_blocks src, blocks
  src = clear_blocks src


generate = (name, templates) ->  
  #
  # Note: Recursion is not catched
  #
  src = templates[name]
  
  
  src = parse_parents src, templates
  src = parse_includes src, templates


# MODELs

TemplateSchema = new mongoose.Schema
  name: String
  src:  String

TemplateSchema.static 'update_or_create', (file, callback) ->
  name = file.name.replace('.html', '')
  src = fs.readFileSync file.path, 'utf-8'

  Template.findOne {name: name}, (err, template) ->
    unless template
      template = new Template
      template.name = name

    template.src = src
    template.save()
    
    callback()


TemplateSchema.static 'render', (name, context={}, callback) ->
  Template.find {}, (err, docs) ->
    templates = {}
    docs.each (d) -> templates[d.name] = d.src
  
    src = generate(name, templates)
  
    try
      html = eco.render src, context
    catch err
      html = "<b>Template:</b> #{err}"
  
    callback(html)


Template = mongoose.model('Template', TemplateSchema)


app.post '/templates', (req, res, next) ->
  req.form.complete (err, fields, files) ->
    if err
      next(err)
    else
      file = files.file
      Template.update_or_create file, ->
        res.send 'ok\n'


app.delete '/templates/:name', (req, res) ->
  Template.findOne {slug: req.params.name}, (err, template) ->
    if template
      template.remove ->
        res.send "ok\n"
    else  
      res.send "not found\n"


module.exports = Template


# Express res.render overriding

http = require('http')
res = http.ServerResponse.prototype

res.render = (name, context={}) ->
  Template.render name, context, (html) =>
    @send html

